using System;
using System.Data;
using System.Configuration;
using System.IO;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Microsoft.Reporting.WebForms;
using WebReporter.Entities;
using RS = WebReporter.ReportService;

public partial class _Default : System.Web.UI.Page
{
    #region Properties
    /// <summary>
    /// Report path
    /// </summary>
    private string ReportPath
    {
        get
        {
            if (radRemote.Checked)
                return String.Format(@"{0}/{1}", txtFolder.Text, cmbReports.Text);
            else
                return String.Format(@"{0}\Reports\{1}.rdlc", Server.MapPath("."), cmbReports.Text);
        }
    }

    /// <summary>
    /// The ReportViewer processing mode
    /// </summary>
    internal ProcessingMode ProcessingMode
    {
        get
        {
            return radRemote.Checked ? ProcessingMode.Remote : ProcessingMode.Local;
        }

    }
    #endregion Properties
    #region Events
    protected void Page_Load(object sender, EventArgs e)
    {
        if (!this.IsPostBack)
        {
            txtServer.Text = WebReporter.Properties.Settings.Default.ReportServerUrl;
            txtFolder.Text = WebReporter.Properties.Settings.Default.ReportPath;
            reportViewer.ServerReport.ReportServerUrl = new Uri(WebReporter.Properties.Settings.Default.ReportServerUrl);
            LoadReports();
        }

    }
    protected void btnRun_Click(object sender, EventArgs e)
    {
        RunReport();
    }
    protected void cmbReports_SelectedIndexChanged(object sender, EventArgs e)
    {

        if (!cmbReports.Enabled) return;
        LoadReportParameters();
    }
    protected void ProcessingMode_Changed(object sender, EventArgs e)
    {
        LoadReports();
    }
    #endregion Events
    #region Remote Processing
    /// <summary>
    /// Loads reports from the report catalog using the ListChildren API.
    /// </summary>
    private void LoadServerReports()
    {
        cmbReports.Items.Clear();

        RS.ReportingService2005 rs = new RS.ReportingService2005();
        rs.Credentials = System.Net.CredentialCache.DefaultCredentials;
        try
        {
            RS.CatalogItem[] items = rs.ListChildren(txtFolder.Text, false);
            foreach (RS.CatalogItem item in items)
            {
                if (item.Type == RS.ItemTypeEnum.Report) cmbReports.Items.Add(item.Name);
            }
            
        }
        catch (Exception ex)
        {
            WebReporter.Util.ShowErrorMessage(String.Empty, ex, lblError);
        }
    }
    /// <summary>
    /// Loads report parameters from the Execution endpoint
    /// </summary>
    private ReportParameterInfoCollection LoadRemoteReportParameters()
    {
        reportViewer.Reset();
        reportViewer.ProcessingMode = ProcessingMode.Remote;
        reportViewer.ServerReport.ReportPath = this.ReportPath;
        // Uncomment the next two lines to localize ReportViewer
        // Done here to avoid "The source of the report definition has not been specified" error
        //ReportViewerCustomMessages myMessageClass = new ReportViewerCustomMessages();
        //reportViewer.Messages = myMessageClass;
        return reportViewer.ServerReport.GetParameters();
    }
    /// <summary>
    /// Runs a remote report
    /// </summary>
    internal void RunRemote()
    {
        try
        {
            //reportViewer.Reset();
            reportViewer.ProcessingMode = ProcessingMode.Remote;
            reportViewer.ServerReport.ReportPath = this.ReportPath;
            // Configure parameters
            ReportParameter[] parameters = UserParametersToReportParameters();
            reportViewer.ServerReport.SetParameters(parameters);
        }
        catch (Exception ex)
        {
            // Normally, we shouldn't reach the catch block, since exceptions are handled by the report viewer
            // and error messages are displayed in the report pane. 
            throw ex;
        }
    }
    #endregion Remote Processing
    #region Local Processing
    /// <summary>
    /// Loads local report files (*.rdlc) from the applicaton startup folder
    /// </summary>
    private void LoadLocalReports()
    {
        cmbReports.Items.Clear();
        string[] reports = Directory.GetFiles(Server.MapPath("Reports"), "customer orders.rdlc");

        foreach (string report in reports)
        {
            cmbReports.Items.Add(Path.GetFileNameWithoutExtension(report));
        }
    }
    /// <summary>
    /// Loads report parameters from the Execution endpoint
    /// </summary>
    private ReportParameterInfoCollection LoadLocalReportParameters()
    {
        reportViewer.Reset();
        reportViewer.ProcessingMode = this.ProcessingMode;
        reportViewer.LocalReport.ReportPath = this.ReportPath;
        reportViewer.LocalReport.EnableExternalImages = true;
        reportViewer.LocalReport.EnableHyperlinks = true;
        return reportViewer.LocalReport.GetParameters();
    }
    /// <summary>
    /// Runs a remote report
    /// </summary>
    internal void RunLocal()
    {
        reportViewer.Reset();
        reportViewer.ProcessingMode = ProcessingMode.Local;
        // Set report path
        reportViewer.LocalReport.ReportPath = this.ReportPath;
        // Enable hyperlinks to handle clicking on the Customer ID
        reportViewer.LocalReport.EnableHyperlinks = true;
        // Required since the logo image in Customer Orders report is external
        reportViewer.LocalReport.EnableExternalImages = true;
        // Required since the Custom Orders report is using a custom function to access the external image
        reportViewer.LocalReport.ExecuteReportInCurrentAppDomain(System.Reflection.Assembly.GetExecutingAssembly().Evidence);
        // Customer Orders references System.Web to get the Web application path so we need to trust it
        reportViewer.LocalReport.AddTrustedCodeModuleInCurrentAppDomain("System.Web, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a");

        // Add report parameters
        ReportParameter[] parameters = this.UserParametersToReportParameters();
        reportViewer.LocalReport.SetParameters(parameters);
        // Bind data
        reportViewer.LocalReport.DataSources.Add(GetCustomers(parameters[0]));
    }

    private ReportDataSource GetCustomers(ReportParameter customers)
    {
        string criteria = null;
        // Get the application dataset. In real life, the dataset would probably come from the business logic layer
        foreach (String value in customers.Values)
        {
            criteria += "," + value;
        }
        criteria = criteria.Remove(0, 1);  // remove trailing ","
        // Need to use a custom adapter to handle the IN clause in the SQL statement
        MyCustomerAdapter adapter = new MyCustomerAdapter(criteria);
        CustomerOrders.CustomerOrdersDataTable table = adapter.GetData();
        // Only one dataset is required for for this report
        return new ReportDataSource("dsOrders", table);
    }

    protected void reportViewer_Drillthrough(object sender, DrillthroughEventArgs e)
    {
        // Handle local drillthrough only
        if (e.Report is ServerReport) return;

        LocalReport localReport = (LocalReport)e.Report;
        // Sales Order Detail local report takes two datasets
        // Load the Sales Order dataset
        WebReporter.Entities.SalesOrderTableAdapters.OrderTableAdapter orderAdapter = new WebReporter.Entities.SalesOrderTableAdapters.OrderTableAdapter();
        SalesOrder.OrderDataTable orderTable = orderAdapter.GetData(localReport.OriginalParametersToDrillthrough[0].Values[0]);
        // Load the Sales Order Detail dataset
        WebReporter.Entities.SalesOrderDetailTableAdapters.OrderDetailTableAdapter orderDetailAdapter = new WebReporter.Entities.SalesOrderDetailTableAdapters.OrderDetailTableAdapter();
        SalesOrderDetail.OrderDetailDataTable orderDetailTable = orderDetailAdapter.GetData(localReport.OriginalParametersToDrillthrough[0].Values[0]);

        localReport.DataSources.Add(new ReportDataSource("SalesOrder", orderTable));
        localReport.DataSources.Add(new ReportDataSource("SalesOrderDetail", orderDetailTable));
    }

    #endregion Local Processing    
    #region Helper Methods
    private void RunReport()
    {
        if (radRemote.Checked)
        {
            RunRemote();
        }
        else
        {
            RunLocal();
        }
    }
    private void LoadReports()
    {
        reportViewer.Reset();
        if (this.ProcessingMode == ProcessingMode.Remote)
            LoadServerReports();
        else
            LoadLocalReports();

        if (cmbReports.Items.Count > 0)
        {
            cmbReports.SelectedIndex = 0;
            LoadReportParameters();
            RunReport();
        }
    }
    private void LoadReportParameters()
    {
        EntityParameter.ParametersDataTable userParameters;
        ReportParameterInfoCollection reportViewerParameters;
        reportViewer.Reset();
        if (this.ProcessingMode == ProcessingMode.Remote)
            reportViewerParameters = LoadRemoteReportParameters();
        else
            reportViewerParameters = LoadLocalReportParameters();

        userParameters = ReportParametersToUserParameters(reportViewerParameters);
        userParameters.DefaultView.AllowNew = false;
        userParameters.DefaultView.AllowDelete = false;
        userParameters.Columns[0].ReadOnly = true;
        grdParams.DataSource = userParameters;
        grdParams.DataBind();
        Session[WebReporter.Util.SESSION_PARAMETERS] = userParameters;
        
    }
    private EntityParameter.ParametersDataTable ReportParametersToUserParameters(ReportParameterInfoCollection reportViewerParameters)
    {
        EntityParameter.ParametersDataTable userParameters = new EntityParameter.ParametersDataTable();
        foreach (ReportParameterInfo p in reportViewerParameters)
        {
            string[] values = new string[p.Values.Count];
            for (int i = 0; i < p.Values.Count; i++)
            {
                values[i] = p.Values[i];
            }
            userParameters.Rows.Add(p.Name, String.Join(",", values));
        }
        return userParameters;
    }
    private ReportParameter[] UserParametersToReportParameters()
    {

        EntityParameter.ParametersDataTable userParameters = (EntityParameter.ParametersDataTable)Session[WebReporter.Util.SESSION_PARAMETERS];
        ReportParameterInfoCollection reportParameters = this.ProcessingMode == ProcessingMode.Remote ?
            reportViewer.ServerReport.GetParameters() : reportViewer.LocalReport.GetParameters();

        ReportParameter[] parameters = new ReportParameter[reportParameters.Count];

        for (int i = 0; i < reportParameters.Count; i++)
        {
            EntityParameter.ParametersRow p = (EntityParameter.ParametersRow)userParameters.Rows[i];
            if (reportParameters[i].MultiValue)
                parameters[i] = new ReportParameter(p.Name, p.Value.Split(new char[] { ',' }));
            else
                parameters[i] = new ReportParameter(p.Name, p.Value);

        }
        return parameters;
    }
    #endregion Helper methods
    #region GridView
    protected void grdParams_RowEditing(object sender, GridViewEditEventArgs e)
    {
        //Set the edit index.
        grdParams.EditIndex = e.NewEditIndex;
        //Bind data to the GridView control.
        BindData();
    }
    private void BindData()
    {
        grdParams.DataSource = Session[WebReporter.Util.SESSION_PARAMETERS];
        grdParams.DataBind();
    }
    protected void grdParams_RowUpdating(object sender, GridViewUpdateEventArgs e)
    {
        //Retrieve the table from the session object.
        DataTable dt = (DataTable)Session[WebReporter.Util.SESSION_PARAMETERS];

        if (dt == null) return;
        //Update the values.
        GridViewRow row = grdParams.Rows[e.RowIndex];
        dt.Rows[grdParams.EditIndex]["Value"] = ((TextBox)(row.Cells[2].Controls[0])).Text;
        //Reset the edit index.
        grdParams.EditIndex = -1;
        //Bind data to the GridView control.
        BindData();
      }
    protected void grdParams_RowCancelingEdit(object sender, GridViewCancelEditEventArgs e)
    {
        //Reset the edit index.
        grdParams.EditIndex = -1;
        //Bind data to the GridView control.
        BindData();
    }
    #endregion GridView
}
