#region Disclaimer by Teo Lachev
/*============================================================================
  File:     AuthenticationExtension.cs

  Summary:  Demonstrates an implementation of an authentication extension.
--------------------------------------------------------------------
  This code sample was built upon on the Microsoft Custom Security sample 
  http://tinyurl.com/33lhoy
 
  The following portions of the code have been changed/added to fit the book needs:
  1. Changed IsValidUser implementation.
    
===========================================================================*/
#endregion

using System;
using System.Data;
using System.Data.SqlClient;
using System.Globalization;
using System.Security.Cryptography;
using System.Text.RegularExpressions;
using System.Web.Security;
using System.Management;
using System.Xml;

namespace Prologika.RS.Extensibility.CustomSecurityExtension
{
   internal sealed class AuthenticationUtilities
   {
      // The path of any item in the report server database 
      // has a maximum character length of 260
      private const int MaxItemPathLength = 260;

      // WMI constants
      private const string _wmiNamespace = @"\\{0}\root\Microsoft\SqlServer\ReportServer\RS_{1}\v10";
      private const string _rsAsmx = @"/ReportService2005.asmx";
	  
       const string			   ROLE_INDIVIDUAL = "individual";
//	   const string			   SP_VALID_USER		= "uspValidateUser";
//	   const string			   SP_VALID_PRINCIPAL	= "uspIsValidPrincipalName";

      // Method used to create a cryptographic random number that is used to
      // salt the user's password for added security
      internal static string CreateSalt(int size)
      {
         // Generate a cryptographic random number using the cryptographic
         // service provider
         RNGCryptoServiceProvider rng = new RNGCryptoServiceProvider();
         byte[] buff = new byte[size];
         rng.GetBytes(buff);
         // Return a Base64 string representation of the random number
         return Convert.ToBase64String(buff);
      }

      // Returns a hash of the combined password and salt value
      internal static string CreatePasswordHash(string pwd, string salt)
      {
         // Concat the raw password and salt value
         string saltAndPwd = String.Concat(pwd, salt);
         // Hash the salted password
         string hashedPwd = 
            FormsAuthentication.HashPasswordForStoringInConfigFile(
            saltAndPwd, "SHA1");
         return hashedPwd;
      }

      // Stores the account details in a SQL table named UserAccounts
      internal static void StoreAccountDetails(string userName,
         string passwordHash,
         string salt)
      {
         // See "How To Use DPAPI (Machine Store) from ASP.NET" for 
         // information about securely storing connection strings.
         SqlConnection conn = new SqlConnection( "Server=localhost;" +
            "Integrated Security=SSPI;" + "database=UserAccounts");

         SqlCommand cmd = new SqlCommand("RegisterUser", conn);
         cmd.CommandType = CommandType.StoredProcedure;
         SqlParameter sqlParam = null;

         sqlParam = cmd.Parameters.Add("@userName", SqlDbType.VarChar, 255);
         sqlParam.Value = userName;

         sqlParam = cmd.Parameters.Add(
            "@passwordHash ", SqlDbType.VarChar, 40);
         sqlParam.Value = passwordHash;

         sqlParam = cmd.Parameters.Add("@salt", SqlDbType.VarChar, 10);
         sqlParam.Value = salt;

         try
         {
            conn.Open();
            cmd.ExecuteNonQuery();
         }
         catch (Exception ex)
         {
            // Code to check for primary key violation (duplicate account 
            // name) or other database errors omitted for clarity
            throw new Exception("An error occurred while attempting to " + 
               "add the account. " + ex.Message);
         }
         finally
         {
            conn.Close();
         } 
      }

	   /// <summary>
	   /// Method that indicates if the user has been authenticated successfully
	   /// </summary>
	   /// <param name="suppliedUserName">The user name</param>
	   /// <param name="suppliedPassword">The user password</param>
	   /// <param name="connectionString">The connection string to the user profile store as specified in RSReportServer.config</param>
	   /// <returns></returns>
	   internal static bool IsValidUser(string suppliedUserName, string suppliedPassword, string connectionString)
	   { 
		   SqlDataReader reader = null;
		   if (suppliedUserName.Trim() == String.Empty) return false;

		   SqlConnection conn = new SqlConnection(connectionString);

		   SqlCommand cmd = new SqlCommand(Constants.SP_VALID_USER , conn);
		   cmd.CommandType = CommandType.StoredProcedure;

		   SqlParameter sqlParam = cmd.Parameters.Add("@uid", SqlDbType.Int, 4);
		   sqlParam.Value = Int32.Parse(suppliedUserName);
		   sqlParam = cmd.Parameters.Add("@pwd", SqlDbType.NVarChar, 50);
		   sqlParam.Value = suppliedPassword;
		   
		   try
		   {
			   conn.Open();
			   reader = cmd.ExecuteReader(CommandBehavior.CloseConnection);
			   return reader.HasRows;
		   }
		   catch (Exception ex)
		   {
			   throw new Exception("An error occurred while attempting to " + 
				   "verify the user. " + ex.Message);
		   }
		   finally
		   {
			   if (reader!=null) reader.Close();
		   }
	   }

      // Method that indicates whether 
      // the supplied username and password are valid
      internal static bool IsValidPrincipalName(string suppliedUserName, string connectionString)
      { 
		  // prevent accidental lockout for admin users
		  if (0 == String.Compare(suppliedUserName, AuthenticationExtension.AdminLogin, true, CultureInfo.CurrentCulture)) return true;

	     SqlDataReader reader = null;
		 if (suppliedUserName.Trim() == String.Empty) return false;

         SqlConnection conn = new SqlConnection(connectionString);

         SqlCommand cmd = new SqlCommand(Constants.SP_VALID_PRINCIPAL , conn);
         cmd.CommandType = CommandType.StoredProcedure;

         SqlParameter sqlParam = cmd.Parameters.Add("@PrincipalName", SqlDbType.NVarChar, 50);
         sqlParam.Value = suppliedUserName;
         try
         {
            conn.Open();
            reader = cmd.ExecuteReader(CommandBehavior.CloseConnection);
            return reader.HasRows;
         }
         catch (Exception ex)
         {
            throw new Exception("An error occurred while attempting to " + 
               "verify the user. " + ex.Message);
         }
         finally
         {
            reader.Close();
         }
      }

      // Method to verify that the user name does not contain any
      // illegal characters. If My Reports is enabled, illegal characters
      // will invalidate the paths created in the \Users folder. Usernames
      // should not contain the characters captured by this method.
      internal static bool ValidateUserName(string input)
      {
         Regex r = new Regex(@"\A(\..*)?[^\. ](.*[^ ])?\z(?<=\A[^/;\?:@&=\+\$,\\\*<>\|""]{0," +
            MaxItemPathLength.ToString() + @"}\z)", 
            RegexOptions.Compiled | RegexOptions.ExplicitCapture);
         bool isValid = r.IsMatch(input);
         return isValid;
      }



      internal static string GetReportServerUrl(string machineName, string instanceName)
      {
          string reportServerVirtualDirectory = String.Empty;
          string fullWmiNamespace = string.Format(_wmiNamespace, machineName, instanceName);

          ManagementScope scope = null;

          ConnectionOptions connOptions = new ConnectionOptions();
          connOptions.Authentication = AuthenticationLevel.PacketPrivacy;

          //Get management scope
          try
          {
              scope = new ManagementScope(fullWmiNamespace, connOptions);
              scope.Connect();

              //Get management class
              ManagementPath path = new ManagementPath("MSReportServer_Instance");
              ObjectGetOptions options = new ObjectGetOptions();
              ManagementClass serverClass = new ManagementClass(scope, path, options);

              serverClass.Get();

              if (serverClass == null)
                  throw new Exception(string.Format(CultureInfo.InvariantCulture,
                    "No WMI class found."));

              //Get instances
              ManagementObjectCollection instances = serverClass.GetInstances();

              foreach (ManagementObject instance in instances)
              {
                  instance.Get();

                  ManagementBaseObject outParams = (ManagementBaseObject)instance.InvokeMethod("GetReportServerUrls",
                     null, null);

                  string[] appNames = (string[])outParams["ApplicationName"];
                  string[] urls = (string[])outParams["URLs"];

                  for (int i = 0; i < appNames.Length; i++)
                  {
                      if (appNames[i] == "ReportServerWebService")
                          reportServerVirtualDirectory = urls[i];
                  }

                  if (reportServerVirtualDirectory == string.Empty)
                      throw new Exception(string.Format(CultureInfo.InvariantCulture,
                         "No Url reservation found for the \"ReportServerWebService\" application."));
              }
          }
          catch (Exception ex)
          {
              throw new Exception(string.Format(CultureInfo.InvariantCulture,
                  "An error occurred while attempting to get the ReportServer Url. " + ex.Message), ex);
          }

          return reportServerVirtualDirectory + _rsAsmx;
      }
	   

   }
}
