#region Disclaimer by Teo Lachev
/*============================================================================
  File:      WebServiceDeliveryProvider.cs

  Summary:  Demonstrates an implementation of a custom delivery extension extension.
--------------------------------------------------------------------
  This code sample was built upon on the Microsoft Printer Delivery Extension Sample 
  http://tinyurl.com/29jthp
 
  The following portions of the code have been changed/added to fit the book needs:
  1. Report delivery to a web service
===========================================================================*/
#endregion
using System;
using System.IO;
using System.Xml;
using System.Collections;
using System.Globalization;
using System.Text;
using System.Reflection;
using Microsoft.ReportingServices.Interfaces;
using Prologika.RS.Extensibility.CustomDeliveryExtension;

namespace Prologika.RS.Extensibility.CustomDeliveryExtension
{
	// A delivery provider must implement IDeliveryExtension and IExtension
	public class WebServiceDeliveryProvider : IDeliveryExtension, IExtension
	{
		// Private member variables used to support the delivery extension
		private bool                              _isPrivilegedUser = false;
		private IDeliveryReportServerInformation  _reportServerInfo = null;
		private Setting[]                         _settings = null;
		private RenderedOutputFile[]              _files;
		StringBuilder                             _sb = new StringBuilder();

		// my variables
		private	string							  _wsdl = null;
		private	string							  _methodName = null;
            
		public WebServiceDeliveryProvider()
		{
			//
			// TODO: Add constructor logic here
			//
		}

		/// <summary>
		/// LocalizedName property
		/// </summary>
		public string LocalizedName
		{
			// Return a localized name based on the user thread culture
			get
			{
				return GetCultureLocalizedName();
			}
		}
      
		/// <summary>
		/// Process XML data stored in the configuration file
		/// </summary>
		/// <param name="configuration">The XML string from the configuration file that contains extension configuration data.</param>
		public void SetConfiguration (string configuration)
		{
			try
			{
				// Create the document and load the Configuration element    
				XmlDocument doc = new XmlDocument();
				doc.LoadXml(configuration);
         
				// Add each setting from the configuration data
				if (doc.DocumentElement.Name == Constants.CONFIG_ROOT)
				{
					foreach (XmlNode child in doc.DocumentElement.ChildNodes)
					{
						switch (child.Name)
						{
							case Constants.CONFIG_WSDL			: this._wsdl = child.InnerText; break;
							case Constants.CONFIG_METHOD_NAME	: this._methodName = child.InnerText; break;
						}
					}
				}
			}

			catch (Exception exception)
			{
				throw new Exception("Failed to retrieve configuration data: " + exception.Message);
			}
		}

		/// <summary>
		/// Validate the user-specified settings.
		/// </summary>
		/// <param name="settings">An array of Setting[] objects containing extension settings supplied by a client.</param>
		/// <returns>The validated settings</returns>
		public Setting[] ValidateUserData(Setting[] settings)
		{

			foreach (Setting setting in settings)
			{
				// If the setting is based on a field, no check is required
				if (setting.Field == null || setting.Field == "")
				{
					switch (setting.Name)
					{
						case (Constants.CONFIG_WSDL):
							if(setting.Value.Trim() == String.Empty)
								setting.Error = String.Format(String.Format("{0} is a required field.", setting.Name));
							break;
						case (Constants.CONFIG_METHOD_NAME):
							if(setting.Value.Trim() == String.Empty)
								setting.Error = String.Format("WSDL is a required field.");
							break;
						default:
							// This is an unknown setting element
							setting.Error = String.Format("Unknown setting {0}", setting.Name);
							break;
					}
				}
			}

			return settings;
		}

		/// <summary>
		/// Required for implementation. Not used by this provider
		/// </summary>
		public bool IsPrivilegedUser
		{
			get
			{
				return _isPrivilegedUser;
			}
			set
			{
				_isPrivilegedUser = value;
			}
		}

		/// <summary>
		/// Returns the valid settings for the WS delivery extension
		/// </summary>
		public Setting[] ExtensionSettings
		{
			get
			{
				if (_settings == null)
				{
					// Setting for the WS WSDL
					_settings = new Setting[2];
					_settings[0]   = new Setting();
					_settings[0].Name = Constants.CONFIG_WSDL;
					_settings[0].ReadOnly = false;
					_settings[0].Required = true;
					_settings[0].Value = this._wsdl;
					_settings[0].AddValidValue(Constants.CONFIG_WSDL, this._wsdl);

					// Setting for the method name
					_settings[1]   = new Setting();
					_settings[1].Name = Constants.CONFIG_METHOD_NAME;
					_settings[1].ReadOnly = false;
					_settings[1].Required = true;
					_settings[1].Value = this._methodName;
					_settings[1].AddValidValue(Constants.CONFIG_METHOD_NAME, this._methodName);
				}
            
				return _settings;
			}
		}
    

		/// <summary>
		/// Sets information about the report server that the delivery extension requires 
		/// in order to perform deliveries. This property contains the names of rendering 
		/// extensions supported by the server. Depending on the functionality of your 
		/// delivery extension, you should limit the available rendering extensions to only 
		/// those supported by your delivery mechanism
		/// </summary>
		public IDeliveryReportServerInformation ReportServerInformation
		{
			set
			{
				_reportServerInfo = value;
			}
		}
        /// <summary>
        /// Returns true if a given extension is supported
        /// </summary>
        /// <param name="extensionName"></param>
        /// <returns></returns>
        private bool IsExtensionSupported (string extensionName)
        {
            foreach (Extension e in _reportServerInfo.RenderingExtension)
                if (e.Name.ToLower() == extensionName.ToLower()) return true;

            return false;
        }

		/// <summary>
		/// Delivers the report notification to a user based on the contents of the notification.
		/// </summary>
		/// <param name="notification">A Notification object containing information required by the 
		/// delivery extension to deliver a report.</param>
		/// <returns>A Boolean value indicating whether or not to retry the delivery.</returns>
		public bool Deliver(Notification notification)
		{
			bool retry = false;

            if (!this.IsExtensionSupported("XML")) throw new Exception("The XML rendering extension is disabled.");

			// Set the status of the notification to pending
			notification.Status = "Processing web service delivery ...";
         
			try
			{
				// Build user data 
				Setting[] userSettings = notification.UserData;
				SubscriptionData subscriptionData = new SubscriptionData();
				subscriptionData.FromSettings(userSettings);
            
				// Deliver the report
				DeliverReport(notification, subscriptionData);

				// If delivery is successful return false
				return retry;
         
			}

			catch (Exception e)
			{
				// Set the status of the notification if an error occurs
				// and attempt retry
				notification.Status = "Error: " + e.Message;
				retry = true;
			}

			finally
			{
				// Finally, save the notification information
				notification.Save();
			}

			return retry;
		}

		/// <summary>
		/// Renders a report and sends it to the web service.
		/// </summary>
		/// <param name="notification">The report notification to use for delivery.</param>
		/// <param name="data">The user data to use for delivery.</param>
		private void DeliverReport(Notification notification, SubscriptionData data)
		{
			_sb.Append(System.DateTime.Now + Environment.NewLine);
			StringWriter stringWriter = null;
			
			try
			{
				// Render report
				_files = notification.Report.Render("XML", @"<DeviceInfo/>");

				if (_files[0].Data.Length > 0)
				{
					// load the report payload to a byte array
					byte[] reportPayload = new byte[_files[0].Data.Length];
					_files[0].Data.Position = 0;
					_files[0].Data.Read(reportPayload, 0, reportPayload.Length);
					_files[0].Data.Flush();
					// use these lines to save the report payload to a file for troubleshooting
//					FileStream fs = new FileStream(@"c:\temp\out.pdf",FileMode.Create);
//					fs.Write(reportPayload,0, reportPayload.Length);
//					fs.Close();
					
					// XML-lize the report payload
					string payload = Convert.ToBase64String(reportPayload);
					StringBuilder stringBuilder = new StringBuilder();
					stringWriter = new StringWriter(stringBuilder);
					XmlTextWriter writer = new XmlTextWriter(stringWriter);
					writer.Formatting = Formatting.Indented;

					writer.WriteStartElement("Report");
					writer.WriteElementString("ReportPayload", payload);
					writer.WriteEndElement();

					// send the report to the requested web service 
					// by passing it as the first parameter
                    string[] parameters = null;
                    WebServiceProxyFactory factory = new WebServiceProxyFactory(data.WSDL);
                    object proxy = factory.Build();
                    Type type = proxy.GetType();

                    MethodInfo method = type.GetMethod(data.methodName);
                    if (method == null || (!CheckMethod(method)))
                    {
                        throw new System.Exception(String.Format("Method {0} is not valid.", method));
                    }
                    parameters = new string[1];
                    parameters[0] = stringBuilder.ToString();
                    string result = method.Invoke(proxy, parameters) as string;

					if (result.ToLower()!="success") throw new Exception(String.Format("Delivering report {0} has resulted in an exception.", notification.Report.Name));

				}
				else
					throw new Exception(String.Format("Rendering report {0} has resulted in no data.", notification.Report));

                     
				// Set the status if the report pages were successfully sent to the web service
				notification.Status = String.Format("Report delivered to {0}", data.WSDL);
			}
         
			catch (Exception exception)
			{
				// Set the status of the notification if an error occurs
				notification.Status = exception.Message;
				// Write exception information to a logfile
				_sb.Append(exception.Message + ": " + exception.StackTrace + Environment.NewLine);
			}   
			finally
			{
				if (stringWriter!=null) stringWriter.Close();
				WriteLog(_sb);
				_sb = null;
			}
		}
	
        private static bool CheckMethod(MethodInfo method)
        {
            // Make sure it's public
            if (!method.IsPublic)
            {
                return false;
            }

            // Make sure it's not static
            if (method.IsStatic)
            {
                return false;
            }

            return true;
        }

    

		// Used for debugging. Currently writes a log file to
		// the system temp directory.
		public void WriteLog(StringBuilder sb)
		{
			try
			{
				FileStream fs = new FileStream(System.IO.Path.GetTempPath() + "WSDeliveryLog.txt", FileMode.Append, 
				FileAccess.Write);
				StreamWriter writer = new StreamWriter(fs);
				writer.Write(sb.ToString());
				writer.Flush();
				writer.Close();
			}

			catch (Exception e)
			{
				throw new IOException("Error writing to log file: " + e.Message);
			}
		}



		// Method to evaluate the culture info of the current thread and return
		// a localized name. This extension name supports German and English names
		// Ideally it would also support a localized Description propery in the UI
		// as well as localized setting labels.
		private string GetCultureLocalizedName()
		{
			switch (CultureInfo.CurrentCulture.Name)
			{
					// TODO: Add more languages or ideally use resource files
				case "de-AT":
				case "de-DE":
				case "de-LI": 
				case "de-LU": 
				case "de-CH":
				default:
					return Constants.EXTENSION_NAME;
			}
		}
	}

}
